/*
 * Tiny Vector Matrix Library
 * Dense Vector Matrix Libary of Tiny size using Expression Templates
 *
 * Copyright (C) 2001 - 2003 Olaf Petzold <opetzold@wit.regiocom.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: BenchImpl.h,v 1.3 2003/02/08 23:14:01 opetzold Exp $
 */

#ifndef TVMET_BENCHIMPL_H
#define TVMET_BENCHIMPL_H

#include <iostream>
#include <vector>
#include <string>

#include <tvmet/util/Timer.h>

#include <Interface.h>

/* forwards */
template<class IfcTag, class T> class Benchmark;
template<class T> class BenchInfo;

/**
 * \class BenchImplBase<class IfcTag, class T>
 * \brief Base class for all interface implementation, regardless of the type.
 * The size is omitted to allow collection benchmarks by operations.
 */
template<class IfcTag, class T>
class BenchImplBase
{
private:
  typedef IfcTag						interface_tag;
  typedef T							value_type;

protected:
  /** Constructor. */
  BenchImplBase(Benchmark<interface_tag, value_type>& benchmark, std::size_t mean_runs)
    : m_benchmark( benchmark ),
      m_mean_runs( mean_runs )
  { }

public:
  /** Virtual Destructor. */
  virtual ~BenchImplBase()
  { }

public: // benchmark interface

  /** Get the timer interface from benchmark. */
  tvmet::util::Timer& timer() { return m_benchmark.timer(); }

  /** Get the BenchInfo interface from benchmark. */
  BenchInfo<T>& bench_info() { return m_benchmark.bench_info(); }

  /** number of mean calculation runs. */
  std::size_t mean_runs() const { return m_mean_runs; }

public: // pure virtual implementation  interface

  /** The name of the implementation libary. */
  virtual std::string name() const = 0;

  /** The size/dimension of the actually benchmark. */
  virtual std::size_t size() const = 0;

public: // (abstract) benchmark math implementions interface

  /** Vector Addition Implementation. */
  virtual void vector_add(std::size_t runs)  { std::cerr << "Error: Benchhmark not implemented!\n"; }

  /** Vector Inner Product Implementation. */
  virtual void vector_inner_product(std::size_t runs)  { std::cerr << "Error: Benchhmark not implemented!\n"; }

  /** Vector Outer Product Implementation. */
  virtual void vector_outer_product(std::size_t runs)  { std::cerr << "Error: Benchhmark not implemented!\n"; }

  /** Matrix Vector Multiply Implementation. */
  virtual void matrix_vector_product(std::size_t runs)  { std::cerr << "Error: Benchhmark not implemented!\n"; }

  /** Matrix Matrix Multiply Implementation. */
  virtual void matrix_matrix_product(std::size_t runs)  { std::cerr << "Error: Benchhmark not implemented!\n"; }

  /** Fortran's AXPY Implementation. */
  virtual void axpy(std::size_t runs)  { std::cerr << "Error: Benchhmark not implemented!\n"; }

public: // (abstract) peak interface

  /** peak addition */
  virtual void scalar_add(std::size_t runs) { std::cerr << "Error: Benchhmark not implemented!\n"; }

  /** peak multiplication */
  virtual void scalar_mul(std::size_t runs) { std::cerr << "Error: Benchhmark not implemented!\n"; }

private: // data
  Benchmark<interface_tag, value_type>&				m_benchmark;
  std::size_t							m_mean_runs;
};


/**
 * \class BenchImpl
 * \brief Empty Benchmark implementation for interface types.
 * These class has to bo specialized for each implementatin type since the
 * construction differs.
 */
template<class IfcTag, class T, std::size_t Sz,
	 class IfcTp = typename InterfaceTraits<IfcTag>::interface_type>
class BenchImpl : public BenchImplBase<IfcTag, T>
{
public:
  typedef IfcTag						interface_tag;
  typedef IfcTp							interface_type;
};


#define XTIMES_ONE 1





/**
 * \class BenchImpl<IfcTag, T, Sz,interface_type1_tag>
 * \brief Benchmark implementation for interface type 1, like tvmet and blitz++
 */
template<class IfcTag, class T, std::size_t Sz>
class BenchImpl<IfcTag, T, Sz, interface_type1_tag>
  : public BenchImplBase<IfcTag, T>
{
private:

public:
  typedef T							value_type;

private:
  typedef IfcTag						interface_tag;
  typedef Interface<interface_tag, value_type, Sz>		interface;

public:
  typedef typename interface::vector_type			vector_type;
  typedef typename interface::matrix_type			matrix_type;

public:
  /** Constructor */
  BenchImpl(Benchmark<interface_tag, value_type>& benchmark, std::size_t mean_runs)
    : BenchImplBase<interface_tag, value_type>( benchmark, mean_runs )
  { }

  virtual ~BenchImpl()
  { }

public: // base class interface

  /** The name of the implementation libary. */
  std::string name() const { return interface::name(); }

  /** The size/dimension of the actually benchmark. */
  std::size_t size() const { return Sz; }

public: // benchmark math implementions

  /** Vector Addition Implementation. */
  void vector_add(std::size_t runs)
  {
    tvmet::util::Timer::time_t					elapsed(0);
    vector_type							x, y;
#if defined(XTIMES_ONE)
    static vector_type						z1(0);
#else
    static vector_type						z1(0), z2(0), z3(0), z4(0), z5(0),
      								z6(0), z7(0), z8(0), z9(0), z10(0);
#endif

    interface::random_fill(x);
    interface::random_fill(y);

    // for mean calculation of the elapsed time, run it more than once
    for(std::size_t r = 0; r != mean_runs(); ++r) {

      timer().restart();

      // the real benchmark loop
      for(std::size_t i = 0; i != runs; ++i) {
#if defined(XTIMES_ONE)
	interface::vector_add(x, y, z1);
#else
	interface::vector_add(x, y, z2);
	interface::vector_add(x, y, z3);
	interface::vector_add(x, y, z4);
	interface::vector_add(x, y, z5);
	interface::vector_add(x, y, z6);
	interface::vector_add(x, y, z7);
	interface::vector_add(x, y, z8);
	interface::vector_add(x, y, z9);
	interface::vector_add(x, y, z10);
#endif
      }

      elapsed += timer().elapsed();

      std::cout << "#" << std::flush;
    }

#ifdef DEBUG
    std::cout << this->size() << " " << mean_runs() << " " << runs << " " << elapsed << std::endl;
#endif

    bench_info().add_result( this->size(), elapsed/mean_runs() );
  }

  /** Vector Inner Product Implementation. */
  void vector_inner_product(std::size_t runs)
  {
    tvmet::util::Timer::time_t					elapsed(0);
    vector_type							x, y;
#if defined(XTIMES_ONE)
    static value_type						z1(0);
#else
    static value_type						z1(0), z2(0), z3(0), z4(0), z5(0),
      								z6(0), z7(0), z8(0), z9(0), z10(0);
#endif

    interface::random_fill(x);
    interface::random_fill(y);

    // for mean calculation of the elapsed time, run it more than once
    for(std::size_t r = 0; r != mean_runs(); ++r) {

      timer().restart();

      // the real benchmark loop
      for(std::size_t i = 0; i != runs; ++i) {
#if defined(XTIMES_ONE)
	interface::vector_inner_product(x, y, z1);
#else
	interface::vector_inner_product(x, y, z2);
	interface::vector_inner_product(x, y, z3);
	interface::vector_inner_product(x, y, z4);
	interface::vector_inner_product(x, y, z5);
	interface::vector_inner_product(x, y, z6);
	interface::vector_inner_product(x, y, z7);
	interface::vector_inner_product(x, y, z8);
	interface::vector_inner_product(x, y, z9);
	interface::vector_inner_product(x, y, z10);
#endif
      }

      elapsed += timer().elapsed();

      std::cout << "#" << std::flush;
    }

#ifdef DEBUG
    std::cout << this->size() << " " << mean_runs() << " " << runs << " " << elapsed << std::endl;
#endif

    bench_info().add_result( this->size(), elapsed/mean_runs() );
  }

  /** Vector Outer Product Implementation. */
  void vector_outer_product(std::size_t runs)
  { }

  /** Matrix Vector Multiply Implementation. */
  void matrix_vector_product(std::size_t runs)
  {
    tvmet::util::Timer::time_t					elapsed(0);
    matrix_type							M;
    vector_type							x;
#if defined(XTIMES_ONE)
    static vector_type						z1(0);
#else
    static vector_type						z1(0), z2(0), z3(0), z4(0), z5(0),
      								z6(0), z7(0), z8(0), z9(0), z10(0);
#endif

    interface::random_fill(M);
    interface::random_fill(x);

    // for mean calculation of the elapsed time, run it more than once
    for(std::size_t r = 0; r != mean_runs(); ++r) {

      timer().restart();

      // the real benchmark loop
      for(std::size_t i = 0; i != runs; ++i) {
#if defined(XTIMES_ONE)
	interface::matrix_vector_product(M, x, z1);
#else
	interface::matrix_vector_product(M, x, z2);
	interface::matrix_vector_product(M, x, z3);
	interface::matrix_vector_product(M, x, z4);
	interface::matrix_vector_product(M, x, z5);
	interface::matrix_vector_product(M, x, z6);
	interface::matrix_vector_product(M, x, z7);
	interface::matrix_vector_product(M, x, z8);
	interface::matrix_vector_product(M, x, z9);
	interface::matrix_vector_product(M, x, z10);
#endif
      }

      elapsed += timer().elapsed();

      std::cout << "#" << std::flush;
    }

#ifdef DEBUG
    std::cout << this->size() << " " << mean_runs() << " " << runs << " " << elapsed << std::endl;
#endif

    bench_info().add_result( this->size(), elapsed/mean_runs() );
  }

  /** Matrix Matrix Multiply Implementation. */
  void matrix_matrix_product(std::size_t runs)
  {
    tvmet::util::Timer::time_t					elapsed(0);
    matrix_type							M1, M2;
#if defined(XTIMES_ONE)
    static matrix_type						Z1;
#else
    static matrix_type						Z1, Z2, Z3, Z4, Z5,
      								Z6, Z7, Z8, Z9, Z10;
#endif

    interface::random_fill(M1);
    interface::random_fill(M2);

    // for mean calculation of the elapsed time, run it more than once
    for(std::size_t r = 0; r != mean_runs(); ++r) {

      timer().restart();

      // the real benchmark loop
      for(std::size_t i = 0; i != runs; ++i) {
#if defined(XTIMES_ONE)
	interface::matrix_matrix_product(M1, M2, Z1);
#else
	interface::matrix_matrix_product(M1, M2, Z2);
	interface::matrix_matrix_product(M1, M2, Z3);
	interface::matrix_matrix_product(M1, M2, Z4);
	interface::matrix_matrix_product(M1, M2, Z5);
	interface::matrix_matrix_product(M1, M2, Z6);
	interface::matrix_matrix_product(M1, M2, Z7);
	interface::matrix_matrix_product(M1, M2, Z8);
	interface::matrix_matrix_product(M1, M2, Z9);
	interface::matrix_matrix_product(M1, M2, Z10);
#endif
      }

      elapsed += timer().elapsed();

      std::cout << "#" << std::flush;
    }

#ifdef DEBUG
    std::cout << this->size() << " " << mean_runs() << " " << runs << " " << elapsed << std::endl;
#endif

    bench_info().add_result( this->size(), elapsed/mean_runs() );
  }

  /** Fortran's AXPY Implementation. */
  void axpy(std::size_t runs)
  {
    tvmet::util::Timer::time_t					elapsed(0);
    vector_type							x;
    value_type							a;
#if defined(XTIMES_ONE)
    static vector_type						y1(0);
#else
    static vector_type						y1(0), y2(0), y3(0), y4(0), y5(0),
      								y6(0), y7(0), y8(0), y9(0), y10(0);
#endif

    interface::random_fill(x);
    a = random<T>(0, 1);

    // for mean calculation of the elapsed time, run it more than once
    for(std::size_t r = 0; r != mean_runs(); ++r) {

      timer().restart();

      // the real benchmark loop
      for(std::size_t i = 0; i != runs; ++i) {
#if defined(XTIMES_ONE)
	interface::axpy(x, a, y1);
#else
	interface::axpy(x, a, y2);
	interface::axpy(x, a, y3);
	interface::axpy(x, a, y4);
	interface::axpy(x, a, y5);
	interface::axpy(x, a, y6);
	interface::axpy(x, a, y7);
	interface::axpy(x, a, y8);
	interface::axpy(x, a, y9);
	interface::axpy(x, a, y10);
#endif
      }

      elapsed += timer().elapsed();

      std::cout << "#" << std::flush;
    }

#ifdef DEBUG
    std::cout << this->size() << " " << mean_runs() << " " << runs << " " << elapsed << std::endl;
#endif

    bench_info().add_result( this->size(), elapsed/mean_runs() );
  }
};





/**
 * \class BenchImpl<IfcTag, T, Sz,interface_type1_tag>
 * \brief Benchmark implementation for interface type 2, like uBlas etc.
 */
template<class IfcTag, class T, std::size_t Sz>
class BenchImpl<IfcTag, T, Sz, interface_type2_tag>
  : public BenchImplBase<IfcTag, T>
{
public:
  typedef T							value_type;

private:
  typedef IfcTag						interface_tag;
  typedef Interface<interface_tag, value_type, Sz>		interface;

public:
  typedef typename interface::vector_type			vector_type;
  typedef typename interface::matrix_type			matrix_type;

public:
  /** Constructor */
  BenchImpl(Benchmark<interface_tag, value_type>& benchmark, std::size_t mean_runs)
    : BenchImplBase<interface_tag, value_type>( benchmark, mean_runs )
  { }

  virtual ~BenchImpl()
  { }

  /** The name of the implementation libary. */
  std::string name() const { return interface::name(); }

  /** The size/dimension of the actually benchmark. */
  std::size_t size() const { return Sz; }

public: // benchmark math implementions

  /** Vector Addition Implementation. */
  void vector_add(std::size_t runs)
  {
    tvmet::util::Timer::time_t					elapsed(0);
    vector_type							x(Sz), y(Sz);
#if defined(XTIMES_ONE)
    static vector_type						z1(Sz);
#else
    static vector_type						z1(Sz), z2(Sz), z3(Sz), z4(Sz), z5(Sz),
								z6(Sz), z7(Sz), z8(Sz), z9(Sz), z10(Sz);
#endif

    interface::random_fill(x);
    interface::random_fill(y);

    // for mean calculation of the elapsed time, run it more than once
    for(std::size_t r = 0; r != mean_runs(); ++r) {

      timer().restart();

      // the real benchmark loop
      for(std::size_t i = 0; i != runs; ++i) {
#if defined(XTIMES_ONE)
	interface::vector_add(x, y, z1);
#else
	interface::vector_add(x, y, z2);
	interface::vector_add(x, y, z3);
	interface::vector_add(x, y, z4);
	interface::vector_add(x, y, z5);
	interface::vector_add(x, y, z6);
	interface::vector_add(x, y, z7);
	interface::vector_add(x, y, z8);
	interface::vector_add(x, y, z9);
	interface::vector_add(x, y, z10);
#endif
      }

      elapsed += timer().elapsed();

      std::cout << "#" << std::flush;
    }

#ifdef DEBUG
    std::cout << this->size() << " " << mean_runs() << " " << runs << " " << elapsed << std::endl;
#endif

    bench_info().add_result( this->size(), elapsed/mean_runs() );
  }

  /** Vector Inner Product Implementation. */
  void vector_inner_product(std::size_t runs)
  {
    tvmet::util::Timer::time_t					elapsed(0);
    vector_type							x(Sz), y(Sz);
#if defined(XTIMES_ONE)
    static value_type						z1(0);
#else
    static value_type						z1(0), z2(0), z3(0), z4(0), z5(0),
								z6(0), z7(0), z8(0), z9(0), z10(0);
#endif

    interface::random_fill(x);
    interface::random_fill(y);

    // for mean calculation of the elapsed time, run it more than once
    for(std::size_t r = 0; r != mean_runs(); ++r) {

      timer().restart();

      // the real benchmark loop
      for(std::size_t i = 0; i != runs; ++i) {
#if defined(XTIMES_ONE)
	interface::vector_inner_product(x, y, z1);
#else
	interface::vector_inner_product(x, y, z2);
	interface::vector_inner_product(x, y, z3);
	interface::vector_inner_product(x, y, z4);
	interface::vector_inner_product(x, y, z5);
	interface::vector_inner_product(x, y, z6);
	interface::vector_inner_product(x, y, z7);
	interface::vector_inner_product(x, y, z8);
	interface::vector_inner_product(x, y, z9);
	interface::vector_inner_product(x, y, z10);
#endif
      }

      elapsed += timer().elapsed();

      std::cout << "#" << std::flush;
    }

#ifdef DEBUG
    std::cout << this->size() << " " << mean_runs() << " " << runs << " " << elapsed << std::endl;
#endif

    bench_info().add_result( this->size(), elapsed/mean_runs() );
  }

  /** Vector Outer Product Implementation. */
  void vector_outer_product(std::size_t runs)
  { }

  /** Matrix Vector Multiply Implementation. */
  void matrix_vector_product(std::size_t runs)
  {
    tvmet::util::Timer::time_t					elapsed(0);
    matrix_type							M(Sz, Sz);
    vector_type							x(Sz);
#if defined(XTIMES_ONE)
    static vector_type						z1(Sz);
#else
    static vector_type						z1(Sz), z2(Sz), z3(Sz), z4(Sz), z5(Sz),
								z6(Sz), z7(Sz), z8(Sz), z9(Sz), z10(Sz);
#endif

    interface::random_fill(M);
    interface::random_fill(x);

    // for mean calculation of the elapsed time, run it more than once
    for(std::size_t r = 0; r != mean_runs(); ++r) {

      timer().restart();

      // the real benchmark loop
      for(std::size_t i = 0; i != runs; ++i) {
#if defined(XTIMES_ONE)
	interface::matrix_vector_product(M, x, z1);
#else
	interface::matrix_vector_product(M, x, z2);
	interface::matrix_vector_product(M, x, z3);
	interface::matrix_vector_product(M, x, z4);
	interface::matrix_vector_product(M, x, z5);
	interface::matrix_vector_product(M, x, z6);
	interface::matrix_vector_product(M, x, z7);
	interface::matrix_vector_product(M, x, z8);
	interface::matrix_vector_product(M, x, z9);
	interface::matrix_vector_product(M, x, z10);
#endif
      }

      elapsed += timer().elapsed();

      std::cout << "#" << std::flush;
    }

#ifdef DEBUG
    std::cout << this->size() << " " << mean_runs() << " " << runs << " " << elapsed << std::endl;
#endif

    bench_info().add_result( this->size(), elapsed/mean_runs() );
  }

  /** Matrix Matrix Multiply Implementation. */
  void matrix_matrix_product(std::size_t runs)
  {
    tvmet::util::Timer::time_t					elapsed(0);
    matrix_type							M1(Sz, Sz), M2(Sz, Sz);
#if defined(XTIMES_ONE)
    static matrix_type						Z1(Sz,Sz);
#else
    static matrix_type						Z1(Sz,Sz), Z2(Sz,Sz), Z3(Sz,Sz), Z4(Sz,Sz), Z5(Sz,Sz),
      								Z6(Sz,Sz), Z7(Sz,Sz), Z8(Sz,Sz), Z9(Sz,Sz), Z10(Sz,Sz);
#endif

    interface::random_fill(M1);
    interface::random_fill(M2);

    // for mean calculation of the elapsed time, run it more than once
    for(std::size_t r = 0; r != mean_runs(); ++r) {

      timer().restart();

      // the real benchmark loop
      for(std::size_t i = 0; i != runs; ++i) {
#if defined(XTIMES_ONE)
	interface::matrix_matrix_product(M1, M2, Z1);
#else
	interface::matrix_matrix_product(M1, M2, Z2);
	interface::matrix_matrix_product(M1, M2, Z3);
	interface::matrix_matrix_product(M1, M2, Z4);
	interface::matrix_matrix_product(M1, M2, Z5);
	interface::matrix_matrix_product(M1, M2, Z6);
	interface::matrix_matrix_product(M1, M2, Z7);
	interface::matrix_matrix_product(M1, M2, Z8);
	interface::matrix_matrix_product(M1, M2, Z9);
	interface::matrix_matrix_product(M1, M2, Z10);
#endif
      }

      elapsed += timer().elapsed();

      std::cout << "#" << std::flush;
    }

#ifdef DEBUG
    std::cout << this->size() << " " << mean_runs() << " " << runs << " " << elapsed << std::endl;
#endif

    bench_info().add_result( this->size(), elapsed/mean_runs() );
  }

  /** Fortran's AXPY Implementation. */
  void axpy(std::size_t runs)
  {
    tvmet::util::Timer::time_t					elapsed(0);
    vector_type							x(Sz);
    value_type							a;
#if defined(XTIMES_ONE)
    static vector_type						y1(Sz);
#else
    static vector_type						y1(Sz), y2(Sz), y3(Sz), y4(Sz), y5(Sz),
      								y6(Sz), y7(Sz), y8(Sz), y9(Sz), y10(Sz);
#endif

    interface::random_fill(x);
    a = random<T>(0, 1);

    // for mean calculation of the elapsed time, run it more than once
    for(std::size_t r = 0; r != mean_runs(); ++r) {

      timer().restart();

      // the real benchmark loop
      for(std::size_t i = 0; i != runs; ++i) {
#if defined(XTIMES_ONE)
	interface::axpy(x, a, y1);
#else
	interface::axpy(x, a, y2);
	interface::axpy(x, a, y3);
	interface::axpy(x, a, y4);
	interface::axpy(x, a, y5);
	interface::axpy(x, a, y6);
	interface::axpy(x, a, y7);
	interface::axpy(x, a, y8);
	interface::axpy(x, a, y9);
	interface::axpy(x, a, y10);
#endif
      }

      elapsed += timer().elapsed();

      std::cout << "#" << std::flush;
    }

#ifdef DEBUG
    std::cout << this->size() << " " << mean_runs() << " " << runs << " " << elapsed << std::endl;
#endif

    bench_info().add_result( this->size(), elapsed/mean_runs() );
  }

};




/**
 * \class BenchImpl<IfcTag, T, Sz, interface_type3_tag>
 * \brief Benchmark implementation for interface type 3 - peak measurement
 */
template<class IfcTag, class T, std::size_t Sz>
class BenchImpl<IfcTag, T, Sz, interface_type3_tag>
  : public BenchImplBase<IfcTag, T>
{
public:
  typedef T							value_type;

private:
  typedef IfcTag						interface_tag;
  typedef Interface<interface_tag, value_type, Sz>		interface;

public:
  /** Constructor */
  BenchImpl(Benchmark<interface_tag, value_type>& benchmark, std::size_t mean_runs)
    : BenchImplBase<interface_tag, value_type>( benchmark, mean_runs )
  { }

  virtual ~BenchImpl()
  { }

  /** The name of the implementation libary. */
  std::string name() const { return interface::name(); }

  /** The size/dimension of the actually benchmark. Size doesn't matter here really. */
  std::size_t size() const { return Sz; }

public: // peak benchmark implementions

  /** peak addition */
  void scalar_add(std::size_t runs)
  {
    tvmet::util::Timer::time_t					elapsed(0);
#if defined(XTIMES_ONE)
    static value_type						s1(0);
#else
    static value_type						s1(0), s2(0), s3(0), s4(0), s5(0),
      								s6(0), s7(0), s8(0), s9(0), s10(0);
#endif

    // for mean calculation of the elapsed time, run it more than once
    for(std::size_t r = 0; r != mean_runs(); ++r) {

      timer().restart();

      // the real benchmark loop
      for(std::size_t i = 0; i != runs; ++i) {
#if defined(XTIMES_ONE)
	interface::scalar_add(s1);
#else
	interface::scalar_add(s2);
	interface::scalar_add(s3);
	interface::scalar_add(s4);
	interface::scalar_add(s5);
	interface::scalar_add(s6);
	interface::scalar_add(s7);
	interface::scalar_add(s8);
	interface::scalar_add(s9);
	interface::scalar_add(s10);
#endif
      }

      elapsed += timer().elapsed();

      std::cout << "#" << std::flush;
    }

#ifdef DEBUG
    std::cout << this->size() << " " << mean_runs() << " " << runs << " " << elapsed << std::endl;
#endif

    bench_info().add_result( this->size(), elapsed/mean_runs() );
  }

  /** peak multiplication */
  void scalar_mul(std::size_t runs)
  {
    tvmet::util::Timer::time_t					elapsed(0);
#if defined(XTIMES_ONE)
    static value_type						s1(1);
#else
    static value_type						s1(1), s2(1), s3(1), s4(1), s5(1),
      								s6(1), s7(1), s8(1), s9(1), s10(1);
#endif

    // for mean calculation of the elapsed time, run it more than once
    for(std::size_t r = 0; r != mean_runs(); ++r) {

      timer().restart();

      // the real benchmark loop
      for(std::size_t i = 0; i != runs; ++i) {
#if defined(XTIMES_ONE)
	interface::scalar_mul(s1);
#else
	interface::scalar_mul(s2);
	interface::scalar_mul(s3);
	interface::scalar_mul(s4);
	interface::scalar_mul(s5);
	interface::scalar_mul(s6);
	interface::scalar_mul(s7);
	interface::scalar_mul(s8);
	interface::scalar_mul(s9);
	interface::scalar_mul(s10);
#endif
      }

      elapsed += timer().elapsed();

      std::cout << "#" << std::flush;
    }

#ifdef DEBUG
    std::cout << this->size() << " " << mean_runs() << " " << runs << " " << elapsed << std::endl;
#endif

    bench_info().add_result( this->size(), elapsed/mean_runs() );
  }

};


#endif /* TVMET_BENCHIMPL_H */

// Local Variables:
// mode:C++
// End:
