#/*
 * Tiny Vector Matrix Library
 * Dense Vector Matrix Libary of Tiny size using Expression Templates
 *
 * Copyright (C) 2001 - 2003 Olaf Petzold <opetzold@wit.regiocom.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: BenchInfo.h,v 1.3 2003/02/08 23:14:01 opetzold Exp $
 */

#ifndef TVMET_BENCHMARK_BENCHINFO_H
#define TVMET_BENCHMARK_BENCHINFO_H

#include <iostream>
#include <vector>
#include <string>
#include <typeinfo>
#include <cmath>						// ceil

#include <boost/shared_ptr.hpp>

#include <tvmet/util/Timer.h>

/**
 * \class BenchInfo
 * \brief Stores all informations from benchmark.
 */
template<class T>
class BenchInfo
{
private:
  class Data
  {
  public:
    Data(std::size_t dim, tvmet::util::Timer::time_t elapsed, double mflops)
      : m_type_info( typeid(T).name() ),
	m_dim( dim ),
	m_elapsed( elapsed ),
	m_mflops( mflops )
    { }
    std::size_t dim() const { return m_dim; }
    tvmet::util::Timer::time_t elapsed() const { return m_elapsed; }
    double mflops() const { return m_mflops; }
  private:
    const char* 						m_type_info;
    std::size_t							m_dim;
    tvmet::util::Timer::time_t					m_elapsed;
    double							m_mflops;
  };

private:
  typedef boost::shared_ptr<Data>				data_pointer;
  typedef std::vector< data_pointer >				data_vector;
  typedef typename data_vector::const_iterator			data_iterator;

private:
  data_vector							m_data;

public:
  /**
   * Constructor. Construct it by implementation+bench name.
   * \param name Name of the Benchmark, e.g. vector_addition.
  */
  BenchInfo(const std::string& name)
    : m_name(name),
      m_bscale(0)
  {  }

  ~BenchInfo()
  {  }

  /** Return the name of the benchmark .*/
  const std::string& bench_name() const { return m_name; }

  /**
   * Set the complexity of the benchmark depends on.
   * \param multplies Complexity of the benchmark multiplies.
   * \param plus Complexity of the benchmark additions.
   * \param runs Number of computations.
   */
  void set_complexity(int multiplies, int plus, int runs)
  {
    m_bscale = static_cast<double>(multiplies * boost::numeric::ublas::type_traits<T>::multiplies_complexity
                          +        plus *       boost::numeric::ublas::type_traits<T>::plus_complexity)
                          * static_cast<double>(runs);

  }

  /** Add the benchmark result as elapsed time */
  void add_result(std::size_t dim, tvmet::util::Timer::time_t elapsed)
  {
    double mflops = m_bscale/(1024.*1024.*static_cast<double>(elapsed));
    m_data.push_back( data_pointer( new Data(dim, elapsed, mflops)));
  }

  /** Simply prints the results. Elapsed/Time vector isn't printed. */
  std::ostream& print_on(std::ostream& os) const
  {
    os << bench_name() << ": ";

    if(m_data.size() > 0) {
      for(data_iterator i = m_data.begin(); i != m_data.end(); ++i) {
	os << static_cast<std::size_t>(ceil(0.5 + (*i)->mflops()))
	   << (i != (m_data.end()-1) ? ", " : "");
      }
    }
    os << " MFlops/s" << std::flush;

    return os;
  }

  /** Simply write prints to filename. */
  void write_to(std::ofstream& file) const
  {

    if(m_data.size() > 0 ) {
      data_iterator 						i( m_data.begin() );

      file << bench_name() << " = [";

      for( ; i != m_data.end()-1; ++i) {
	file << (*i)->dim() << ", " << (*i)->mflops() << "; ";
      }
      file << (*i)->dim() << ", " << (*i)->mflops()
	   << "];\n" << std::flush;
    }
  }

private:
  std::string							m_name;
  double							m_bscale;
};

template<class T>
std::ostream& operator<<(std::ostream& os, const BenchInfo<T>& info) {
  return info.print_on(os);
}


#endif /* TVMET_BENCHMARK_BENCHINFO_H */

// Local Variables:
// mode:C++
// End:
