/*
 * Tiny Vector Matrix Library
 * Dense Vector Matrix Libary of Tiny size using Expression Templates
 *
 * Copyright (C) 2001 - 2003 Olaf Petzold <opetzold@wit.regiocom.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: Benchmark.h,v 1.3 2003/02/08 23:14:01 opetzold Exp $
 */

#ifndef TVMET_BENCHMARK_H
#define TVMET_BENCHMARK_H

#include <iostream>
#include <vector>
#include <string>

#include <boost/shared_ptr.hpp>
#include <boost/numeric/ublas/iterator.hpp>
#include <boost/numeric/ublas/traits.hpp>

#include <BenchImpl.h>
#include <BenchmarkBase.h>


/**
 * \class Benchmark
 * \brief The real benchmark.
 * This benchmark runs different implementations of libaries using the appropriate
 * tag and type. At construction time you can determine the number of repeatition
 * to calculate the mean of average time used by the benchmark. This increase the
 * accuracy for the implemented benchmarked operations. The operation benchmarked
 * self you can run x times using the appropriate member function.
 */
template<class IfcTag, class T>
class Benchmark
  : public BenchmarkBase<T>
{
private:
  typedef IfcTag						interface_tag;

public:
  typedef T							value_type;

private:
  typedef boost::shared_ptr< BenchImplBase<interface_tag, value_type> > benchmark_pointer;
  typedef std::vector< benchmark_pointer > 			benchmark_vector;
  typedef typename benchmark_vector::const_iterator 		benchmark_iterator;

  typedef Interface<interface_tag, value_type, 0>		interface;

private:
  enum {
    xtimes = 1		/**< Number of implementation runs on each benchmark. */
  };

public:

  /**
   * Constructor
   * \param med_run Number of repeation of benchmark runs.
   */
  Benchmark(std::size_t mean_runs = 1)
  {
    m_benchmarks.push_back( benchmark_pointer(new BenchImpl<interface_tag, value_type, 2>(*this, mean_runs)) );
    m_benchmarks.push_back( benchmark_pointer(new BenchImpl<interface_tag, value_type, 3>(*this, mean_runs)) );
    m_benchmarks.push_back( benchmark_pointer(new BenchImpl<interface_tag, value_type, 4>(*this, mean_runs)) );
    m_benchmarks.push_back( benchmark_pointer(new BenchImpl<interface_tag, value_type, 5>(*this, mean_runs)) );
#if 1
    m_benchmarks.push_back( benchmark_pointer(new BenchImpl<interface_tag, value_type, 6>(*this, mean_runs)) );
    m_benchmarks.push_back( benchmark_pointer(new BenchImpl<interface_tag, value_type, 7>(*this, mean_runs)) );
    m_benchmarks.push_back( benchmark_pointer(new BenchImpl<interface_tag, value_type, 8>(*this, mean_runs)) );
    m_benchmarks.push_back( benchmark_pointer(new BenchImpl<interface_tag, value_type, 9>(*this, mean_runs)) );
    m_benchmarks.push_back( benchmark_pointer(new BenchImpl<interface_tag, value_type, 10>(*this, mean_runs)) );
    m_benchmarks.push_back( benchmark_pointer(new BenchImpl<interface_tag, value_type, 11>(*this, mean_runs)) );
    m_benchmarks.push_back( benchmark_pointer(new BenchImpl<interface_tag, value_type, 12>(*this, mean_runs)) );
    m_benchmarks.push_back( benchmark_pointer(new BenchImpl<interface_tag, value_type, 13>(*this, mean_runs)) );
    m_benchmarks.push_back( benchmark_pointer(new BenchImpl<interface_tag, value_type, 14>(*this, mean_runs)) );
    m_benchmarks.push_back( benchmark_pointer(new BenchImpl<interface_tag, value_type, 15>(*this, mean_runs)) );
#endif

    std::cout << "*** Running benchmark for " << libname() << " ***"
	      << std::endl << std::flush;
  }

  /** Destructor, cleans up internal memory. */
  virtual ~Benchmark()
  {  }

public: // implementation interface

  /** Return the name of the libary used for these benchmark. The name doesn't
      depend on size, therefore we can use simple one as proxy. */
  const std::string libname() const { return m_benchmarks.front()->name(); }

public: // benchmark math implementions

  /** Vector Addition Implementation. */
  void vector_add(std::size_t runs)
  {
    if(interface::implements::vector_add) {

      std::cout << "Running vector addition " << std::endl << std::flush;

      register_benchinfo("vector_add");

      if(m_benchmarks.size() != 0) {
	for(benchmark_iterator i = m_benchmarks.begin(); i != m_benchmarks.end(); ++i) {

	  std::size_t sz = (*i)->size();

	  bench_info().set_complexity(0, xtimes*sz, runs );

	  std::cout << "[" << sz << ":" << std::flush;

	  (*i)->vector_add(runs);

	  std::cout << "]" << std::flush;
	}
      }

      std::cout << " (done)." << std::endl << std::flush;
    }
  }

  /** Vector Inner Product Implementation. */
  void vector_inner_product(std::size_t runs)
  {
    if(interface::implements::vector_inner_product) {

      std::cout << "Running vector inner product " << std::endl << std::flush;

      register_benchinfo("vector_inner_product");

      if(m_benchmarks.size() != 0) {
	for(benchmark_iterator i = m_benchmarks.begin(); i != m_benchmarks.end(); ++i) {

	  std::size_t sz = (*i)->size();

	  bench_info().set_complexity(xtimes*sz, xtimes*(sz-1), runs );

	  std::cout << "[" << sz << ":" << std::flush;

	  (*i)->vector_inner_product(runs);

	  std::cout << "]" << std::flush;
	}
      }

      std::cout << " (done)." << std::endl << std::flush;
    }
  }

  /** Vector Outer Product Implementation. */
  void vector_outer_product(std::size_t runs)
  {
    if(interface::implements::vector_outer_product) {

      std::cout << "Running vector outer product " << std::endl << std::flush;

      register_benchinfo("vector_outer_product");

      if(m_benchmarks.size() != 0) {
	for(benchmark_iterator i = m_benchmarks.begin(); i != m_benchmarks.end(); ++i) {

	  std::size_t sz = (*i)->size();

	  bench_info().set_complexity(xtimes*sz*sz, 0, runs);

	  std::cout << "[" << sz << ":" << std::flush;

	  (*i)->vector_outer_product(runs);

	  std::cout << "]" << std::flush;
	}
      }

      std::cout << " (done)." << std::endl << std::flush;
    }
  }

  /** Matrix Vector Multiply Implementation. */
  void matrix_vector_product(std::size_t runs)
  {
    if(interface::implements::matrix_vector_product) {

      std::cout << "Running matrix vector product " << std::endl << std::flush;

      register_benchinfo("matrix_vector_product");

      if(m_benchmarks.size() != 0) {
	for(benchmark_iterator i = m_benchmarks.begin(); i != m_benchmarks.end(); ++i) {

	  std::size_t sz = (*i)->size();

	  bench_info().set_complexity( xtimes*sz*sz, xtimes*sz*(sz-1), runs );

	  std::cout << "[" << sz << ":" << std::flush;

	  (*i)->matrix_vector_product(runs);

	  std::cout << "]" << std::flush;
	}
      }

      std::cout << " (done)." << std::endl << std::flush;
    }
  }

  /** Matrix Matrix Multiply Implementation. */
  void matrix_matrix_product(std::size_t runs)
  {
    if(interface::implements::matrix_matrix_product) {
      std::cout << "Running matrix matrix product " << std::endl << std::flush;

      register_benchinfo("matrix_matrix_product");

      if(m_benchmarks.size() != 0) {
	for(benchmark_iterator i = m_benchmarks.begin(); i != m_benchmarks.end(); ++i) {

	  std::size_t sz = (*i)->size();

	  bench_info().set_complexity( xtimes*sz*sz*sz, xtimes*sz*sz*(sz-1), runs );

	  std::cout << "[" << sz << ":" << std::flush;

	  (*i)->matrix_matrix_product(runs);

	  std::cout << "]" << std::flush;
	}
      }

      std::cout << " (done)." << std::endl << std::flush;
    }
  }

  /** Fortran's AXPY Implementation. */
  void axpy(std::size_t runs)
  {
    if(interface::implements::axpy) {
      std::cout << "Running axpy " << std::endl << std::flush;

      register_benchinfo("axpy");

      if(m_benchmarks.size() != 0) {
	for(benchmark_iterator i = m_benchmarks.begin(); i != m_benchmarks.end(); ++i) {

	  std::size_t sz = (*i)->size();

	  bench_info().set_complexity(xtimes*sz, xtimes*sz, runs);

	  std::cout << "[" << sz << ":" << std::flush;

	  (*i)->axpy(runs);

	  std::cout << "]" << std::flush;
	}
      }

      std::cout << " (done)." << std::endl << std::flush;
    }
  }

public: // peak benchmark implementions

  /**  peak addition */
  void scalar_add(std::size_t runs)
  {
    if(interface::implements::scalar_add) {
      std::cout << "Running peak addition " << std::endl << std::flush;

      register_benchinfo("scalar_add");

      if(m_benchmarks.size() != 0) {
	for(benchmark_iterator i = m_benchmarks.begin(); i != m_benchmarks.end(); ++i) {

	  std::size_t sz = (*i)->size();

	  bench_info().set_complexity(0, xtimes*1 /* scalar! */, runs);

	  std::cout << "[" << sz << ":" << std::flush;

	  (*i)->scalar_add(runs);

	  std::cout << "]" << std::flush;
	}
      }

      std::cout << " (done)." << std::endl << std::flush;
    }
  }

  /**  peak multiplication */
  void scalar_mul(std::size_t runs)
  {
    if(interface::implements::scalar_mul) {
      std::cout << "Running peak multiplication " << std::endl << std::flush;

      register_benchinfo("scalar_mul");

      if(m_benchmarks.size() != 0) {
	for(benchmark_iterator i = m_benchmarks.begin(); i != m_benchmarks.end(); ++i) {

	  std::size_t sz = (*i)->size();

	  bench_info().set_complexity(xtimes*1 /* scalar! */, 0, runs);

	  std::cout << "[" << sz << ":" << std::flush;

	  (*i)->scalar_mul(runs);

	  std::cout << "]" << std::flush;
	}
      }

      std::cout << " (done)." << std::endl << std::flush;
    }
  }

private:
  benchmark_vector						m_benchmarks;
};


#endif /* TVMET_BENCHMARK_H */

// Local Variables:
// mode:C++
// End:
