/*
 * Tiny Vector Matrix Library
 * Dense Vector Matrix Libary of Tiny size using Expression Templates
 *
 * Copyright (C) 2001 - 2003 Olaf Petzold <opetzold@wit.regiocom.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: BenchmarkBase.h,v 1.3 2003/02/08 23:14:01 opetzold Exp $
 */

#ifndef TVMET_BENCHMARK_BENCHMARKBASE_H
#define TVMET_BENCHMARK_BENCHMARKBASE_H

#include <iostream>
#include <vector>
#include <string>
#include <fstream>

#include <boost/shared_ptr.hpp>
#include <boost/numeric/ublas/traits.hpp>

#include <BenchInfo.h>

/**
 * \class BenchmarkBase
 * \brief Base class for benchmarks, holding all basic data and functions.
 */
template<class T>
class BenchmarkBase
{
public:
  typedef T							value_type;

private:
  typedef boost::shared_ptr< BenchInfo<value_type> >		benchinfo_pointer;
  typedef std::vector< benchinfo_pointer > 			benchinfo_vector;
  typedef typename benchinfo_vector::const_iterator 		benchinfo_iterator;

public:

  /** Constructor */
  BenchmarkBase()
  { }

  /** Destructor, cleans up internal memory. */
  virtual ~BenchmarkBase()
  {  }

public: // timer interface

  /** Get the timer interface. */
  tvmet::util::Timer& timer() { return m_timer; }

public: // implementation interface

  virtual const std::string libname() const = 0;

  /** Get the actually BenchInfo. */
  BenchInfo<T>& bench_info()
  {
    assert( m_benchinfos.size() != 0 );
    return *m_benchinfos.back();
  }

public:

  /** Get the results. */
  std::ostream& print_on(std::ostream& os) const
  {
    for(benchinfo_iterator i = m_benchinfos.begin(); i != m_benchinfos.end(); ++i) {
      os << **i << std::flush << std::endl;
    }
    return os;
  }

  /** Save the results to file, formatted as octave/matlab. */
  bool save(const std::string& fname = "") const
  {
    std::ofstream						file;
    std::string							filename(fname);

    if( fname == "")
      filename = this->libname() + ".m";

    file.open( filename.c_str(), std::ios_base::out );

    if( !file.is_open() ) return false;

    for(benchinfo_iterator i = m_benchinfos.begin(); i != m_benchinfos.end(); ++i) {
      (*i)->write_to(file);
    }

    file.close();

    return true;
  }

  /** Register Benchmark by name. */
  void register_benchinfo(const std::string& bench)
  {
    m_benchinfos.push_back( benchinfo_pointer(new BenchInfo<T>(libname() + "_" + bench)) );
  }


private:
  benchinfo_vector						m_benchinfos;

  tvmet::util::Timer 						m_timer;
};

template<class T>
std::ostream& operator<<(std::ostream& os, const BenchmarkBase<T>& bench)
{
  return bench.print_on(os);
}


#endif /* TVMET_BENCHMARK_BENCHMARKBASE_H */

// Local Variables:
// mode:C++
// End:
