/*
 * Tiny Vector Matrix Library
 * Dense Vector Matrix Libary of Tiny size using Expression Templates
 *
 * Copyright (C) 2001 - 2003 Olaf Petzold <opetzold@wit.regiocom.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: Interface.h,v 1.3 2003/02/08 23:14:01 opetzold Exp $
 */

#ifndef TVMET_BENCHMARK_INTERFACE_H
#define TVMET_BENCHMARK_INTERFACE_H

/** forward */
int rand(void);

/**
 * \fn random
 * \brief simply call random using a given range.
 */
template<class T>
T random(T min, T max)
{
  return min + static_cast<T>( max * std::rand()/(RAND_MAX + min) );
}

/** libary tags */
struct interface_tvmet_tag { };
struct interface_blitz_tag { };
struct interface_ublas_tag { };
struct interface_mtl_tag { };
struct interface_stl_tag { };
struct interface_peak_tag { };		// impl. differs completly from others!



/*
 * \class Interface<class Tag, class T, std::size_t Sz>
 * \brief Implementation details.
 */
template<class IfcTag, class T, std::size_t Sz>
class Interface
{
public: // benchmark math implementions

  /** What's implemented here. */
  struct implements {
    enum {
      vector_add = false,
      vector_inner_product = false,
      vector_outer_product = false,
      matrix_vector_product = false,
      matrix_matrix_product = false,
      axpy = false,

      scalar_add = false,
      scalar_mul = false
    };
  };
};


/** This tag is used for implementations like tvmet and blitz++, where the
    dimension is bounded at compile time. */
struct interface_type1_tag { };

/** This tag is used for implementations like ublas etc., where the
    dimension is bounded at run time. */
struct interface_type2_tag { };

/** This tag is used for peak performance implementations, dimensions are not
    of interest here. */
struct interface_type3_tag { };





/**
 * \class InterfaceTraits
 * \brief Libary Interfaces differs, therefore we use traits.
 * This class have to be specialized for (at least) two different interface
 * types. First, implementations like tvmet and blitz++, where the
 *  dimension is bounded at compile time. Second, implementations like
 *  ublas etc., where the dimension is bounded at run time.
 * \sa interface_type1_tag
 * \sa interface_type2_tag
 */
template<class IfcTag>
struct InterfaceTraits
{ };


/**
 * \class InterfaceTraits<interface_tvmet_tag>
 */
template<>
struct InterfaceTraits<interface_tvmet_tag>
{
  typedef interface_tvmet_tag					interface_tag;
  typedef interface_type1_tag					interface_type;
};

/**
 * \class InterfaceTraits<interface_blitz_tag>
 */
template<>
struct InterfaceTraits<interface_blitz_tag>
{
  typedef interface_blitz_tag					interface_tag;
  typedef interface_type1_tag					interface_type;
};

/**
 * \class InterfaceTraits<interface_ublas_tag>
 */
template<>
struct InterfaceTraits<interface_ublas_tag>
{
  typedef interface_ublas_tag					interface_tag;
  typedef interface_type2_tag					interface_type;
};

/**
 * \class InterfaceTraits<interface_mtl_tag>
 */
template<>
struct InterfaceTraits<interface_mtl_tag>
{
  typedef interface_mtl_tag					interface_tag;
  typedef interface_type2_tag					interface_type;
};

/**
 * \class InterfaceTraits<interface_stl_tag>
 */
template<>
struct InterfaceTraits<interface_stl_tag>
{
  typedef interface_stl_tag					interface_tag;
  typedef interface_type2_tag					interface_type;
};

/**
 * \class InterfaceTraits<interface_peak_tag>
 */
template<>
struct InterfaceTraits<interface_peak_tag>
{
  typedef interface_peak_tag					interface_tag;
  typedef interface_type3_tag					interface_type;
};

#endif /* TVMET_BENCHMARK_INTERFACE_H */

// Local Variables:
// mode:C++
// End:
