/*
 * Tiny Vector Matrix Library
 * Dense Vector Matrix Libary of Tiny size using Expression Templates
 *
 * Copyright (C) 2001 - 2003 Olaf Petzold <opetzold@wit.regiocom.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: blitz.h,v 1.4 2003/03/01 06:50:30 opetzold Exp $
 */

#ifndef TVMET_BENCHMARK_BLITZ_H
#define TVMET_BENCHMARK_BLITZ_H

#include <algorithm>


#include <blitz/tinyvec-et.h>
#include <blitz/tinymat.h>

/*
 * \class Interface<interface_blitz_tag, class T, std::size_t Sz>
 * \brief Implementation details.
 */
template<class T, std::size_t Sz>
class Interface<interface_blitz_tag, T, Sz>
{
public:
  typedef T							value_type;

  typedef typename blitz::TinyMatrix<value_type, Sz, Sz>	matrix_type;
  typedef typename blitz::TinyVector<value_type, Sz>		vector_type;

  /** The name of the libary. */
  static inline std::string name() { return "blitz"; }

public:

  /** Fill the given matrix container. */
  static inline void random_fill(matrix_type& M, T min = 0, T max = 1)
  {
    for(std::size_t r = 0; r < Sz; ++r) {
      for(std::size_t c = 0; c < Sz; ++c) {
	M(r, c) = random<T>(min, max);
      }
    }
  }

  /** Fill the given vector container. */
  static inline void random_fill(vector_type& V, T min = 0, T max = 1)
  {
    for(std::size_t i = 0; i < Sz; ++i) {
      V[i] = random<T>(min, max);
    }
  }

public: // benchmark math implementions

  /** What's implemented here. */
  struct implements {
    enum {
      vector_add = true,
      vector_inner_product = true,
      vector_outer_product = false,
      matrix_vector_product = false,
      matrix_matrix_product = false,
      axpy = true,

      scalar_add = false,
      scalar_mul = false
    };
  };

  /** Vector Addition Implementation. */
  static inline void vector_add(const vector_type& x, const vector_type& y, vector_type& z)
  {
    z = x + y;
  }

  /** Vector Inner Product Implementation. */
  static inline void vector_inner_product(const vector_type& x, const vector_type& y, value_type& z)
  {
    z = dot(x, y);
  }

  /** Vector Outer Product Implementation. */
  static inline void vector_outer_product(const vector_type& x, const vector_type& y, vector_type& z)
  {
    z = cross(x, y);
  }

  /** Matrix Vector Multiply Implementation. */
  static inline void matrix_vector_product(const matrix_type& M, const vector_type& x, vector_type& z)
  {
    // not supported
  }

  /** Matrix Matrix Multiply Implementation. */
  static inline void matrix_matrix_product(const matrix_type& M1, const matrix_type& M2, matrix_type& Z)
  {
    // not supported
  }

  /** Fortran's AXPY Implementation. */
  static inline void axpy(const vector_type& x, value_type a, vector_type& y)
  {
     y += a * x;
  }
};

#endif /* TVMET_BENCHMARK_BLITZ_H */

// Local Variables:
// mode:C++
// End:
