/*
 * Tiny Vector Matrix Library
 * Dense Vector Matrix Libary of Tiny size using Expression Templates
 *
 * Copyright (C) 2001 - 2003 Olaf Petzold <opetzold@wit.regiocom.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: stl.h,v 1.3 2003/02/08 23:14:01 opetzold Exp $
 */

#ifndef TVMET_BENCHMARK_STL_H
#define TVMET_BENCHMARK_STL_H

#include <algorithm>
#include <vector>

/*
 * \class Interface<interface_stl_tag, class T, std::size_t Sz>
 * \brief Implementation details.
 */
template<class T, std::size_t Sz>
class Interface<interface_stl_tag, T, Sz>
{
public:
  typedef T							value_type;

  typedef typename std::vector<value_type>			vector_type;
  typedef typename std::vector<vector_type>			matrix_type;

  /** The name of the libary. */
  static inline std::string name() { return "stl"; }

public:

  /** Fill the given container. */
  static inline void random_fill(vector_type& c, T min = 0, T max = 1)
  {
    std::fill( c.begin(), c.end(), random<T>(min, max) );
  }

  /** Fill the given container. */
  static inline void random_fill(matrix_type& M, T min = 0, T max = 1)
  {
    for(typename matrix_type::iterator i = M.begin(); i != M.end(); ++i)
      std::fill( (*i).begin(), (*i).end(), random<T>(min, max) );
  }

public: // benchmark math implementions

  /** What's implemented here. */
  struct implements {
    enum {
      vector_add = true,
      vector_inner_product = false,
      vector_outer_product = false,
      matrix_vector_product = true,
      matrix_matrix_product = true,
      axpy = true,

      scalar_add = false,
      scalar_mul = false
    };
  };

  /** Vector Addition Implementation. */
  static inline void vector_add(const vector_type& x, const vector_type& y, vector_type& z)
  {
    for(std::size_t i = 0 ; i != Sz; ++i)
      z[i] = x[i] + y[i];
  }

  /** Vector Inner Product Implementation. */
  static inline void vector_inner_product(const vector_type& x, const vector_type& y, value_type& z)
  {
    // not supported
  }

  /** Vector Outer Product Implementation. */
  static inline void vector_outer_product(const vector_type& x, const vector_type& y, vector_type& z)
  {
    // not supported
  }

  /** Matrix Vector Multiply Implementation. */
  static inline void matrix_vector_product(const matrix_type& M, const vector_type& x, vector_type& z)
  {
    value_type							s(0);

    for(std::size_t r = 0 ; r != Sz; ++r) {
      s = 0;
      for(std::size_t c = 0 ; c != Sz; ++c) {
	s += M[r][c] * x[c];
      }
      z[r] = s;
    }
  }

  /** Matrix Matrix Multiply Implementation. */
  static inline void matrix_matrix_product(const matrix_type& M1, const matrix_type& M2, matrix_type& Z)
  {
    value_type							s(0);

    for(std::size_t i = 0 ; i != Sz; ++i) {
      for(std::size_t j = 0 ; j != Sz; ++j) {
	s = 0;
	for(std::size_t k = 0 ; k != Sz; ++k) {
	  s += M1[i][k] * M2[k][j];
	}
	Z[i][j] = s;
      }
    }
  }

  /** Fortran's AXPY Implementation. */
  static inline void axpy(const vector_type& x, value_type a, vector_type& y)
  {
     for(std::size_t i = 0 ; i != Sz; ++i) {
       y[i] += a * x[i];
     }
  }
};

#endif // TVMET_BENCHMARK_STL_H
