/*
 * Tiny Vector Matrix Library
 * Dense Vector Matrix Libary of Tiny size using Expression Templates
 *
 * Copyright (C) 2001 - 2003 Olaf Petzold <opetzold@wit.regiocom.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: ublas.h,v 1.3 2003/02/08 23:14:01 opetzold Exp $
 */

#ifndef TVMET_BENCHMARK_UBLAS_H
#define TVMET_BENCHMARK_UBLAS_H

#include <algorithm>

#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/matrix_expression.hpp> // outer_product

/*
 * \class Interface<interface_ublas_tag, class T, std::size_t Sz>
 * \brief Implementation details.
 */
template<class T, std::size_t Sz>
class Interface<interface_ublas_tag, T, Sz>
{
public:
  typedef T							value_type;

#if 0
  typedef typename boost::numeric::ublas::matrix<value_type>	matrix_type;
  typedef typename boost::numeric::ublas::vector<value_type>	vector_type;
#else
  typedef typename boost::numeric::ublas::c_matrix<value_type, Sz, Sz>	matrix_type;
  typedef typename boost::numeric::ublas::c_vector<value_type, Sz>	vector_type;
#endif

  /** The name of the libary. */
  static inline std::string name() { return "ublas"; }

public:

  /** Fill the given matrix container. */
  static inline void random_fill(matrix_type& M, T min = 0, T max = 1)
  {
    for(std::size_t r = 0; r < Sz; ++r) {
      for(std::size_t c = 0; c < Sz; ++c) {
	M(r, c) = random<T>(min, max);
      }
    }
  }

  /** Fill the given vector container. */
  static inline void random_fill(vector_type& V, T min = 0, T max = 1)
  {
    std::fill( V.begin(), V.end(), random<T>(min, max) );
  }

public: // benchmark math implementions

  /** What's implemented here. */
  struct implements {
    enum {
      vector_add = true,
      vector_inner_product = true,
      vector_outer_product = false,
      matrix_vector_product = true,
      matrix_matrix_product = true,
      axpy = true,

      scalar_add = false,
      scalar_mul = false
    };
  };

  /** Vector Addition Implementation. */
  static inline void vector_add(const vector_type& x, const vector_type& y, vector_type& z)
  {
    // z = x + y; assume alias free
    z.assign( x + y );
  }

  /** Vector Inner Product Implementation. */
  static inline void vector_inner_product(const vector_type& x, const vector_type& y, value_type& z)
  {
    z = inner_prod(x, y);
  }

  /** Vector Outer Product Implementation. uBlas does have a completly another Interface as others. ! */
  static inline void vector_outer_product(const vector_type& x, const vector_type& y, matrix_type& z)
  {
    // z = outer_prod(x, y); assume alias free
    z.assign( outer_prod(x, y) );
  }

  /** Matrix Vector Multiply Implementation. */
  static inline void matrix_vector_product(const matrix_type& M, const vector_type& x, vector_type& z)
  {
    // z = prod(M, x); assume alias free
    z.assign( prod(M, x) );
  }

  /** Matrix Matrix Multiply Implementation. */
  static inline void matrix_matrix_product(const matrix_type& M1, const matrix_type& M2, matrix_type& Z)
  {
    // Z = prod(M1, M2); assume alias free
    Z.assign( prod(M1, M2) );
  }

  /** Fortran's AXPY Implementation. */
  static inline void axpy(const vector_type& x, value_type a, vector_type& y)
  {
    // y += a * x; assume alias free
     y.plus_assign( a * x );
  }
};

#endif /* TVMET_BENCHMARK_UBLAS_H */

// Local Variables:
// mode:C++
// End:
