/*
 * Tiny Vector Matrix Library
 * Dense Vector Matrix Libary of Tiny size using Expression Templates
 *
 * Copyright (C) 2001 - 2003 Olaf Petzold <opetzold@wit.regiocom.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: TestEval.h,v 1.1 2003/05/24 07:31:31 opetzold Exp $
 */

#ifndef TVMET_TEST_EVAL_H
#define TVMET_TEST_EVAL_H

#include <cppunit/extensions/HelperMacros.h>

#include <tvmet/Vector.h>
#include <tvmet/Matrix.h>

#include <cassert>

template <class T>
class TestEval : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE( TestEval );
  CPPUNIT_TEST( Greater );
  CPPUNIT_TEST( Less );
  CPPUNIT_TEST( GreaterOrEqual );
  CPPUNIT_TEST( LessOrEqual );
  CPPUNIT_TEST( Equal );
  CPPUNIT_TEST( NotEqual );
  CPPUNIT_TEST( LogicalAnd );
  CPPUNIT_TEST( LogicalOr );
  CPPUNIT_TEST_SUITE_END();

private:
  typedef tvmet::Vector<T, 3>			vector_type;
  typedef tvmet::Matrix<T, 3, 3>		matrix_type;

public:
  TestEval()
    : vOne(1), vZero(0), mOne(1), mZero(0) { }

public: // cppunit interface
  /** cppunit hook for fixture set up. */
  void setUp();

  /** cppunit hook for fixture tear down. */
  void tearDown();

protected:
  void Greater();
  void Less();
  void GreaterOrEqual();
  void LessOrEqual();
  void Equal();
  void NotEqual();
  void LogicalAnd();
  void LogicalOr();

private:
  vector_type v1;
  vector_type vBig;	/**< vector bigger than v1 */
  const vector_type vOne;
  const vector_type vZero;

private:
  matrix_type m1;
  matrix_type mBig;	/**< matrix bigger than m1 */
  const matrix_type mOne;
  const matrix_type mZero;
};

/*****************************************************************************
 * Implementation Part I (cppunit part)
 ****************************************************************************/

template <class T>
void TestEval<T>::setUp () {
  v1 = 1,2,3;
  vBig = 10,20,30;

  m1 = 1,4,7,
       2,5,8,
       3,6,9;
  mBig = 10,40,70,
         20,50,80,
         30,60,90;
}

template <class T>
void TestEval<T>::tearDown() { }

/*****************************************************************************
 * Implementation Part II
 * these are elemental - therefore we use std::assert
 ****************************************************************************/

/*
 * on SelfTest, we have the guarantee, that the container holds the
 * expected values. Now check comparing operation using tvmet's
 * eval function. This is the basic for all further test since it's
 * the way we check the correctness. The other way would be element wise
 * compare as in SelfTest, urgh...
 */
template <class T>
void
TestEval<T>::Greater() {
  // all test are element wise !
  assert( eval(vBig >  v1) );
  assert( eval(mBig >  m1) );
}

template <class T>
void
TestEval<T>::Less() {
  // all test are element wise !
  assert( eval(v1   <  vBig) );
  assert( eval(m1   <  mBig) );
}

template <class T>
void
TestEval<T>::GreaterOrEqual() {
  // all test are element wise !
  assert( eval(vBig >= v1) );
  assert( eval(v1   >= v1) );
  assert( eval(vBig >= vBig) );
  assert( eval(vOne >= T(1)) );
  assert( eval(vZero>= T(0)) );

  assert( eval(mBig >= m1) );
  assert( eval(m1   >= m1) );
  assert( eval(mBig >= mBig) );
  assert( eval(mOne >= T(1)) );
  assert( eval(mZero>= T(0)) );
}

template <class T>
void
TestEval<T>::LessOrEqual() {
  // all test are element wise !
  assert( eval(v1   <= vBig) );
  assert( eval(v1   <= v1) );
  assert( eval(vBig <= vBig) );
  assert( eval(vOne <= T(1)) );
  assert( eval(vZero<= T(0)) );

  assert( eval(m1   <= mBig) );
  assert( eval(m1   <= m1) );
  assert( eval(mBig <= mBig) );
  assert( eval(mOne <= T(1)) );
  assert( eval(mZero<= T(0)) );
}

template <class T>
void
TestEval<T>::Equal() {
  // all test are element wise !
  assert( eval(v1   == v1) );
  assert( eval(vBig == vBig) );
  assert( eval(vOne == 1) );
  assert( eval(vZero == T(0)) );

  assert( eval(m1   == m1) );
  assert( eval(mBig == mBig) );
  assert( eval(mOne == T(1)) );
  assert( eval(mZero == T(0)) );
}

template <class T>
void
TestEval<T>::NotEqual() {
  // all test are element wise !
  assert( eval(v1   != vBig) );
  assert( eval(m1   != mBig) );
}

template <class T>
void
TestEval<T>::LogicalAnd() {
  // TODO: implement
}

template <class T>
void
TestEval<T>::LogicalOr() {
  // TODO: implement
}


#endif // TVMET_TEST_EVAL_H

// Local Variables:
// mode:C++
// End:
