#include <iostream>

#if !defined(HAVE_TVMET)

#if defined(__GCC__)
#define restrict __restrict__
#else
#define restrict
#endif

/**
 * limited traits
 */
template<class T1, class T2>
struct PromoteTraits { };

struct PromoteTraits<double, double> {
  typedef double					value_type;
};

/**
 * Xpr (I)
 */
class XprNull
{
public:
  explicit XprNull() { }
};

template <class T> inline T operator+(const T& lhs, XprNull) { return lhs; }	

/**
 * Meta
 */
template<size_t Rows1, size_t Cols1, 
	 size_t Cols2, 
	 size_t RowStride1, size_t ColStride1, 
	 size_t RowStride2, size_t ColStride2, 
	 size_t K>
class MetaMM {
  enum { doIt = (K != Cols1 - 1) };
public:
  template<class T1, class T2>
  static inline 
  typename PromoteTraits<T1, T2>::value_type
  product(const T1* restrict lhs, const T2* restrict rhs, size_t i, size_t j) {
    return lhs[i * RowStride1 + K * ColStride1] * rhs[K * RowStride2 + j * ColStride2]
      + MetaMM<Rows1 * doIt, Cols1 * doIt,
               Cols2 * doIt, RowStride1 * doIt, ColStride1 * doIt,
               RowStride2 * doIt, ColStride2 * doIt, (K+1) * doIt>::product(lhs, rhs, i, j);
  }
};

template<>
class MetaMM<0,0,0,0,0,0,0,0> {
public:
  static inline 
  XprNull product(const void*, const void*, size_t, size_t) { return XprNull(); }
};

template<size_t Rows, size_t Cols, size_t RowStride, size_t ColStride,
	 size_t VecStride, size_t J>
class MetaMV {
  enum { doIt = J < (Cols-1) };
public:   
  template<class T1, class T2> 
  static inline 
  typename PromoteTraits<T1, T2>::value_type
  product(const T1* restrict m, const T2* restrict v, size_t i) {
    return m[i * RowStride + J * ColStride] * v[J * VecStride]
      + MetaMV<Rows * doIt, Cols * doIt,
               RowStride * doIt, ColStride * doIt, 
               VecStride * doIt, (J+1)*doIt>::product(m, v, i);
  }
};

template<>
class MetaMV<0,0,0,0,0,0> {
public:
  static inline 
  XprNull product(const void*, const void*, size_t) { return XprNull(); }
};

/**
 * Xpr (II)
 */
template<class T>
class XprLiteral {
public:
  typedef T						value_type;
  explicit XprLiteral(value_type rhs) : data_(rhs) { }
  XprLiteral(const  XprLiteral<value_type>& rhs) : data_(rhs.data_) { }
  value_type operator[](size_t) const { return data_; }
  value_type operator()(size_t, size_t) const { return data_; }
private:
  value_type data_;
};

template<class E, size_t Sz>
class XprVector {
public:
  typedef typename E::value_type			value_type;
public:
  explicit XprVector(E expr) : e_(expr) { }
  value_type operator[](size_t i) const { return e_[i]; }
private:
  E e_;
};

template<class E, size_t Rows, size_t Cols>
class XprMatrix {
public:
  typedef typename E::value_type			value_type;
public:
  explicit XprMatrix(E expr) : e_(expr) { }
  XprMatrix(const XprMatrix<E, Rows, Cols>& expr) : e_(expr.e_) { }
  value_type operator()(size_t i, size_t j) const { return e_(i, j); }
private:
  E e_;
};

template<class BinOp, class E1, class E2>
class XprBinOp {
public:
  typedef typename 
  PromoteTraits<typename E1::value_type, 
		typename E2::value_type>::value_type	value_type;
public:
  explicit XprBinOp(E1 lhs, E2 rhs) : e1_(lhs), e2_(rhs) { }
  XprBinOp(const XprBinOp<BinOp, E1, E2>& bop) : e1_(bop.e1_), e2_(bop.e2_) { }
  value_type operator[](size_t i) const { return BinOp::applyOn(e1_[i], e2_[i]); }
  value_type operator()(size_t i, size_t j) const { return BinOp::applyOn(e1_(i, j), e2_(i, j)); }
private:
  E1 e1_;
  E2 e2_;
};

template<class T1, size_t Rows1, size_t Cols1,
	 class T2, size_t Cols2, 
	 size_t RowStride1, size_t ColStride1,
	 size_t RowStride2, size_t ColStride2>
class XprMMProduct {
public:
  typedef typename PromoteTraits<T1, T2>::value_type	value_type;
public:
  explicit XprMMProduct(const T1* lhs,const T2* rhs)
    : lhs_(lhs), rhs_(rhs) { }
  XprMMProduct(const XprMMProduct<T1, Rows1, Cols1, 
	       T2, Cols2, 
	       RowStride1, ColStride1,
	       RowStride2, ColStride2>& mmp)
    : lhs_(mmp.lhs_), rhs_(mmp.rhs_) { }
  value_type operator()(size_t i, size_t j) const {
    return MetaMM<Rows1, Cols1, Cols2, 
                  RowStride1, ColStride1, 
                  RowStride2, ColStride2, 0>::product(lhs_, rhs_, i, j);
  }
private:
  const T1* restrict lhs_;
  const T2* restrict rhs_;    
};

template<class T1, size_t Rows, size_t Cols, 
	 size_t RowStride, size_t ColStride, 
	 class T2, size_t VecStride>
class XprMVProduct {
public:
  typedef typename PromoteTraits<T1, T2>::value_type	value_type;
public:    
  explicit XprMVProduct(const T1* matrix, const T2* vector)
    : matrix_(matrix), vector_(vector) { }
  XprMVProduct(const XprMVProduct<T1, Rows, Cols, RowStride, ColStride, T2, VecStride>& mvp)
    : matrix_(mvp.matrix_), vector_(mvp.vector_) { }
  value_type operator[](size_t i) const {
    return MetaMV<Rows, Cols, RowStride,
                  ColStride, VecStride, 0>::product(matrix_, vector_, i);
  }
private:
  const T1* restrict matrix_;
  const T2* restrict vector_;
};

/**
 * Functionals
 */
template <class T1, class T2>
struct fcnl_Assign {
  typedef void						return_type;
  static inline return_type applyOn(T1& restrict lhs, T2 rhs) {
    lhs = static_cast<T1>(rhs);
  }
};

/**
 * Matrix
 */ 
template<class T, size_t Rows, size_t Cols> class Matrix;
template<class T, size_t Rows, size_t Cols, 
	 size_t RowStride, size_t ColStride> class MatrixReference;

template<class T, 
	 size_t Rows, size_t Cols, 
	 size_t RowStride=Cols, size_t ColStride=1>
class MatrixReference {
public: // types
  typedef T						value_type;
  typedef Matrix<T, Rows, Cols>				matrix_type;
  typedef MatrixReference<T, Rows, Cols,
		          RowStride, ColStride>		reference_type;
  explicit MatrixReference(const matrix_type& rhs)
    : data_(rhs.data()) { }
  MatrixReference(const reference_type& rhs)
    : data_(rhs.data_) { }
  value_type operator()(size_t i, size_t j) const {
    return data_[i * RowStride + j * ColStride]; 
  }
private:
  const value_type* restrict data_;

};

template<size_t Rows, size_t Cols, 
	 size_t RowStride, size_t ColStride>
class MetaMatrix {
  enum {
    doRows = (RowStride < Rows - 1) ? 1 : 0,	/**< recursive counter Rows. */
    doCols = (ColStride < Cols - 1) ? 1 : 0	/**< recursive counter Cols. */
  };
public:
  typedef MetaMatrix<Rows, Cols, RowStride, ColStride>	metamatrix_type;
  template<class Matrx, class Expr, class Operator>
  static inline 
  void assign2(Matrx& mat, Expr expr, Operator op) {
    op.applyOn(mat(RowStride,ColStride), expr(RowStride,ColStride));
    MetaMatrix<Rows * doCols, Cols * doCols, 
               RowStride * doCols, (ColStride+1) * doCols>::assign2(mat, expr, op);
  }
  template<class Matrx, class Expr, class Operator>
  static inline 
  void assign(Matrx& mat, Expr expr, Operator op) {
    MetaMatrix<Rows, Cols, 
               RowStride, 0>::assign2(mat, expr, op);
    MetaMatrix<Rows * doRows, Cols * doRows, 
               (RowStride+1) * doRows, 0>::assign(mat, expr, op);
  }
};

template<>
class MetaMatrix<0, 0, 0, 0> {
public:
  template<class Matrx, class Expr, class Operator>
  static inline void assign2(Matrx&, Expr, Operator) { }
  template<class Matrx, class Expr, class Operator>
  static inline void assign(Matrx&, Expr, Operator) { }
};

template<class T, size_t Rows, size_t Cols>
class Matrix {
public:
  typedef T						value_type;
  typedef Matrix<T, Rows, Cols>				matrix_type;
public:
  ~Matrix() { }
  explicit Matrix() { }
  Matrix(const matrix_type& rhs) {
    *this = XprMatrix<typename matrix_type::const_reference, Rows, Cols>(rhs.ref()); 
  }
  explicit Matrix(value_type rhs) {
    typedef XprLiteral<value_type> expr_type;
    *this = XprMatrix<expr_type, Rows, Cols>(expr_type(rhs));
  }
  template<class E>
  explicit Matrix(XprMatrix<E, Rows, Cols> expr) {
    assign(expr, fcnl_Assign<value_type, typename E::value_type>());
  }
  value_type* restrict data() { return data_; }
  const value_type* restrict data() const { return data_; }
  value_type& restrict operator()(size_t i, size_t j) {
    return data_[i * Cols + j]; 
  }
  value_type operator()(size_t i, size_t j) const {
    return data_[i * Cols + j]; 
  }
  typedef MatrixReference<T, Rows, Cols>   		const_reference;
  const_reference ref() const { return const_reference(*this); }
  template<class E, class Op>
  void assign(E expr, Op op) {
    MetaMatrix<Rows, Cols, 0, 0>::assign(*this, expr, op);
  }
  template <class E> matrix_type& operator=(XprMatrix<E, Rows, Cols> expr) {
    assign(expr, fcnl_Assign<value_type, typename E::value_type>());
    return *this;
  }
  std::ostream& printOn(std::ostream& os) const {
    os << "Matrix<" << Rows << ", " << Cols << ">(\n";
    for(size_t i = 0; i < Rows; ++i) {
      for(size_t j = 0; j < Cols; ++j) {
	os << this->operator()(i, j);
	if( j != (Cols - 1) ) os << ", ";
      }
      if( i != (Rows - 1) ) os << ";\n ";
    }
    os << ")";
    return os;
  }
private:
  value_type data_[Rows * Cols];
};

template<class T, size_t Rows, size_t Cols>
inline
std::ostream& operator<<(std::ostream& os, const Matrix<T, Rows, Cols>& rhs) {
  return rhs.printOn(os);
}

/**
 * Vector
 */
template<class T, size_t Sz> class Vector;
template<class T, size_t Sz, size_t Stride> class VectorReference;

template<class T, size_t Sz, size_t Stride=1>
class VectorReference {
public:
  typedef T 						value_type;
  typedef Vector<T, Sz>					vector_type;
  typedef VectorReference<T, Sz, Stride>		reference_type;
  explicit VectorReference(const vector_type& rhs): data_(rhs.data()) { }
  VectorReference(const reference_type& rhs) : data_(rhs.data_) { }
  value_type operator[](size_t i) const { return data_[i * Stride]; }
private:
  const value_type* restrict data_;
};

template<size_t Sz, size_t Idx=0>
class MetaVector {
  enum { doIt = (Idx < (Sz-1)) ? 1 : 0 };
public:
  typedef MetaVector<Sz, Idx>			metavector_type;
  template <class Vec, class Expr, class Operator>
  static inline 
  void assign(Vec& vec, Expr expr, Operator op) {
    op.applyOn(vec[Idx], expr[Idx]);
    MetaVector<Sz * doIt, (Idx+1) * doIt>::assign(vec, expr, op);
  }
};

/**
 * MetaVector Specialized for recursion
 */
template<>
class MetaVector<0,0> {
public:
  template <class Vec, class Expr, class Operator>
  static inline void assign(Vec&, Expr, Operator) { }
};

template<class T, size_t Sz>
class Vector {
  enum { size = Sz };
public:
  typedef T     					value_type;
  typedef Vector<T, Sz>					vector_type;
  ~Vector() { }
  explicit Vector() { }
  Vector(const vector_type& rhs) {
    *this = XprVector<typename vector_type::const_reference, Sz>(rhs.ref()); 
  }
  explicit Vector(value_type rhs) {
    typedef XprLiteral<value_type> expr_type;
    *this = XprVector<expr_type, Sz>(expr_type(rhs));
  }
  explicit Vector(value_type x0, value_type x1, value_type x2) {
    data_[0] = x0; data_[1] = x1; data_[2] = x2;
  }
  value_type* restrict data() { return data_; }
  const value_type* restrict data() const { return data_; }
  value_type& restrict operator[](size_t i) { return data_[i]; }
  value_type operator[](size_t i) const { return data_[i]; }
  typedef VectorReference<T, Sz, 1>    			const_reference;
  const_reference ref() const { return const_reference(*this); }
  template<class E, class Op>
  void assign(E expr, Op op) {
    MetaVector<Sz>::assign(*this, expr, op);
  }
  template <class E> vector_type& operator=(XprVector<E, Sz> expr) {
    assign(expr, fcnl_Assign<value_type, typename E::value_type>());
    return *this;
  }
  std::ostream& printOn(std::ostream& os) const {
    os << "Vector<" << Sz << ">(";
    for(size_t i = 0; i < Sz; ++i) {
      os << data_[i];
      if( i != (Sz - 1) ) os << ", ";
    }
    os << ")";
    return os;
  }
private:
  value_type data_[Sz];
};

template <class T, size_t Sz>
inline
std::ostream& operator<<(std::ostream& os, const Vector<T, Sz>& rhs) {
  return rhs.printOn(os);
}

/**
 * Operators and Functions
 */
template<class T1, size_t Rows1, size_t Cols1,
	 class T2, size_t Cols2>
inline
XprMatrix<
  XprMMProduct<
    T1, Rows1, Cols1, 
    T2, Cols2, Cols1,
    1, Cols2, 1
  >,
  Rows1, Cols2
>
product(const Matrix<T1, Rows1, Cols1>& lhs,
	const Matrix<T2, Cols1, Cols2>& rhs) {
  typedef XprMMProduct<
    T1, Rows1, Cols1, 
    T2, Cols2, Cols1, 
    1, Cols2, 1
  >							expr_type;
  return XprMatrix<expr_type, Rows1, Cols2>(expr_type(lhs.data(), rhs.data()));
}

template<class T1, size_t Rows1, size_t Cols1,
	 class T2, size_t Cols2>
inline
XprMatrix<
  XprMMProduct<
    T1, Rows1, Cols1, 
    T2, Cols2, Cols1,
    1, Cols2, 1
  >,
  Rows1, Cols2
>
operator*(const Matrix<T1, Rows1, Cols1>& lhs,
	  const Matrix<T2, Cols1, Cols2>& rhs) {
  return product(lhs, rhs);
}

#else

#include <tvmet/Vector.h>
#include <tvmet/Matrix.h>

using namespace tvmet;

#endif // HAVE_TVMET








/************** How to write it without temporaries ? *********************/

#if 1 // is working with temporaries

template<class E, size_t Rows, size_t Cols, class T2>
inline
XprVector<
  XprMVProduct<typename E::value_type, Rows, Cols, 
	       Cols, 1, 
	       T2, 1>,
  Rows
>
product(XprMatrix<E, Rows, Cols> lhs, const Vector<T2, Cols>& rhs) {
  // promote 
  Matrix<T2, Rows, Cols> m;
  m = lhs;
  typedef XprMVProduct<typename E::value_type, Rows, Cols, Cols, 1, T2, 1> expr_type;
  return XprVector<expr_type, Rows>(expr_type(m.data(), rhs.data()));
}

template<class E, size_t Rows, size_t Cols, class T2>
inline
XprVector<
  XprMVProduct<typename E::value_type, Rows, Cols, Cols, 1, T2, 1>,
  Rows
>
operator*(XprMatrix<E, Rows, Cols> lhs, const Vector<T2, Cols>& rhs) {
  return product(lhs, rhs);
}

#else

template<class E, size_t Rows, size_t Cols, class T2>
inline
XprVector<
  XprMVProduct<typename E::value_type, Rows, Cols, 
	       Cols, 1, 
	       T2, 1>,
  Rows
>
product(XprMatrix<E, Rows, Cols> lhs, const Vector<T2, Cols>& rhs) {
  return XprVector<expr_type, Rows>(expr_type(m.data(), rhs.data()));
}

#endif

/************************* Test *****************************************/

using namespace std;

typedef Matrix<double, 3,3> matrix33d;
typedef Vector<double, 3>   vector3d;

void test_mmv(vector3d& res, const matrix33d& m1, const matrix33d& m2, const vector3d& v) {
  //res = m1 * m2 * v;
  cout << m1*v   << endl;
}

void test_mmm(matrix33d& res, const matrix33d& m1, const matrix33d& m2,  const matrix33d& m3) {
  //res = m1 * m2 * m3;
}

int main()
{
  using namespace std;

  matrix33d m1, m2, m3, mr;
  m1(0,0) = 1; m1(1,0) = 4; m1(2,0) = 7;
  m1(0,1) = 2; m1(1,1) = 5; m1(2,1) = 8;
  m1(0,2) = 3; m1(1,2) = 6; m1(2,2) = 9;
  m2 = m1;
  m3 = m1;
  
  test_mmm(mr, m1, m2, m3);	// miss operator*(XprMatrix<E> e, const Vector<T, Sz>& v)
  cout << mr << endl;

  vector3d vr(0), v(1,2,3);
  
  test_mmv(vr, m1, m2, v);	// miss operator*(XprMatrix<E> e, const Matrix<T, Rows, Cols>)
  cout << vr << endl;
}

